#[repr(u8)]pub enum Error {
Rejected = 4,
Sealed = 8,
SpuriousFailure = 12,
NotRegistered = 16,
WrongMode = 20,
NotReady = 24,
}Expand description
Errors that can occur when accessing a Gate.
Variants
Rejected = 4
The Gate rejected the task.
Sealed = 8
The Gate has been sealed.
SpuriousFailure = 12
Spurious failure to enter a Gate in a Controlled state.
This can happen if a task holding a Pager gets cancelled or drops the Pager before
the Gate has permitted or rejected the task; the task causes all other waiting tasks
of the Gate to get this error.
NotRegistered = 16
WrongMode = 20
The wrong asynchronous/synchronous mode was used for a Pager.
NotReady = 24
The result is not ready.